@@ -8,8 +8,9 @@ use Cpanel::JSON::XS;
 use JSON::Validator;
 use String::Random;
 use DateTime;
+use Hash::Merge qw/ merge /;
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 has _validator  => sub { JSON::Validator->new };
 has _str_rand   => sub { String::Random->new };
@@ -34,8 +35,8 @@ sub json_schema_to_json {
 	$self->_validator->schema( $schema );
 	$schema = $self->_validator->schema->data;
 
-	my $method = $self->_guess_method( $schema );
-	return $self->$method( $schema );
+	my ( $method,$sub_schema ) = $self->_guess_method( $schema );
+	return $self->$method( $sub_schema );
 }
 
 sub _example_from_spec {
@@ -77,7 +78,7 @@ sub _random_integer {
 	my @possible_values = defined $min && defined $max
 		? $min .. $max
 		: defined $min
-			? $min .. $min + ( $min * $min )
+			? $min .. $min + 1000
 			: defined $max
 				? 1 .. $max
 				: defined $mof
@@ -96,7 +97,7 @@ sub _random_integer {
 		);
 		return $possible_values[0];
 	} else {
-		return $possible_values[ int( rand( $#possible_values + 1 ) ) ];
+		return $self->_random_element( [ @possible_values ] );
 	}
 }
 
@@ -106,6 +107,9 @@ sub _random_number {
 	return $self->_example_from_spec( $schema )
 		if scalar $self->_example_from_spec( $schema );
 
+	return $self->_random_integer( $schema )
+		if ( $schema->{multipleOf} );
+
 	return $self->_random_integer( $schema ) + $self->_random_integer( $schema ) / 10;
 }
 
@@ -116,7 +120,7 @@ sub _random_string {
 		if scalar $self->_example_from_spec( $schema );
 
 	if ( my @enum = @{ $schema->{enum} // [] } ) {
-		return $enum[ int( rand( $#enum + 1 ) ) ];
+		return $self->_random_element( [ @enum ] );
 	}
 
 	return $self->_str_rand->randregex( $schema->{pattern} )
@@ -124,7 +128,13 @@ sub _random_string {
 
 	if ( my $format = $schema->{format} ) {
 		return {
-			"date-time" => DateTime->now->iso8601 . '.000Z',
+			"date-time" => DateTime->now->subtract(
+				weeks => $self->_random_integer({ minimum => 1, maximum => 500 }),
+				days => $self->_random_integer({ minimum => 1 }),
+				hours => $self->_random_integer({ minimum => 1 }),
+				minutes => $self->_random_integer({ minimum => 1 }),
+				seconds => $self->_random_integer({ minimum => 1 }),
+			)->iso8601 . '.000Z',
 			"email"     =>
 				$self->_random_string( { pattern => '[A-Za-z]{12}' } )
 				. '@'
@@ -136,8 +146,12 @@ sub _random_string {
 				maximum => 254,
 			}) } 1 .. 4 ),
 			"ipv6"      => '2001:0db8:0000:0000:0000:0000:1428:57ab',
-			"uri"       => 'https://www.google.com',
-			"uriref"    => 'https://www.google.com',
+			"uri"       => 'https://www.'
+				. $self->_random_string( { pattern => '[a-z]{12}' } )
+				. '.com',
+			"uriref"    => 'https://www.'
+				. $self->_random_string( { pattern => '[a-z]{12}' } )
+				. '.com',
 		}->{ $format };
 	}
 
@@ -193,8 +207,8 @@ sub _random_array {
 sub _add_next_array_item {
 	my ( $self,$array,$schema,$unique ) = @_;
 
-	my $method = $self->_guess_method( $schema );
-	my $value = $self->$method( $schema );
+	my ( $method,$sub_schema ) = $self->_guess_method( $schema );
+	my $value = $self->$method( $sub_schema );
 
 	if ( ! $unique ) {
 		push( @{ $array },$value );
@@ -232,6 +246,11 @@ sub _random_object {
 	my $max = $schema->{maxProperties}
 		|| ( $schema->{minProperties} ? $schema->{minProperties} + 1 : undef );
 
+	if ( ! $min && ! $max ) {
+		# no min or max, just make use of all properties
+		%properties = map { $_ => 1 } keys( %{ $schema->{properties} } );
+	}
+
 	if ( $min && scalar( keys( %properties ) ) < $min ) {
 		# we have too few properties
 		if ( $max ) {
@@ -265,8 +284,9 @@ sub _random_object {
 
 	foreach my $property ( keys %properties ) {
 
-		my $method = $self->_guess_method( $schema->{properties}{$property} );
-		$object->{$property} = $self->$method( $schema->{properties}{$property} );
+		my ( $method,$sub_schema )
+			= $self->_guess_method( $schema->{properties}{$property} );
+		$object->{$property} = $self->$method( $sub_schema );
 	}
 
 	return $object;
@@ -274,18 +294,85 @@ sub _random_object {
 
 sub _random_null { undef }
 
+sub _random_enum {
+	my ( $self,$schema ) = @_;
+	return $self->_random_element( $schema->{'enum'} );
+}
+
 sub _guess_method {
 	my ( $self,$schema ) = @_;
 
-	if ( ref( $schema->{'type'} ) eq 'ARRAY' ) {
-		$schema->{'type'} = $schema->{'type'}->[
-			int( rand( scalar( @{ $schema->{'type'} } ) ) )
-		];
+	if (
+		$schema->{'type'}
+		&& ref( $schema->{'type'} ) eq 'ARRAY'
+	) {
+		$schema->{'type'} = $self->_random_element( $schema->{'type'} );
+	}
+
+	# check for combining schemas
+	if ( my $any_of = $schema->{'anyOf'} ) {
+
+		# easy, pick a random sub schema
+		my $sub_schema = $self->_random_element( $any_of );
+		return $self->_guess_method( $sub_schema );
+
+	} elsif ( my $all_of = $schema->{'allOf'} ) {
+
+		# easy? mush these all together and assume the schema doesn't
+		# contain any contradictory information. note the mushing
+		# together needs to be a little bit smart to prevent stomping
+		# on any duplicate keys (hence Hash::Merge)
+		my $merged_schema = {};
+
+		foreach my $sub_schema ( @{ $all_of } ) {
+			$merged_schema = merge( $merged_schema,$sub_schema );
+		}
+
+		return $self->_guess_method( $merged_schema );
+
+	} elsif ( my $one_of = $schema->{'oneOf'} ) {
+
+		# difficult - we need to generate data that validates against
+		# one and *only* one of the rules, so here we make a poor
+		# attempt and just go by the first rule
+		warn __PACKAGE__ . " encountered oneOf, see CAVEATS perldoc section";
+		return $self->_guess_method( $one_of->[0] );
+
+	} elsif ( my $not = $schema->{'not'} ) {
+
+		if ( my $not_type = $not->{'type'} ) {
+
+			my $type = {
+				"string"  => "integer",
+				"integer" => "string",
+				"number"  => "string",
+				"enum"    => "string",
+				"boolean" => "string",
+				"null"    => "integer",
+				"object"  => "string",
+				"array"   => "object",
+			}->{ $not_type };
+
+			return $self->_guess_method( { type => $type } );
+		}
+
+		# well i don't like this, because by implication it means
+		# the data can be anything but the listed one so it seems
+		# very handwavy in some cases.
+		warn __PACKAGE__ . " encountered not, see CAVEATS perldoc section";
 	}
 
 	# danger danger! accessing private method from elsewhere
-	my $schema_type = JSON::Validator::_guess_schema_type( $schema ) // 'null';
-	return "_random_$schema_type";
+	my $schema_type = JSON::Validator::_guess_schema_type( $schema );
+
+	$schema_type //= 'null';
+
+	return ( "_random_$schema_type",$schema );
+}
+
+sub _random_element {
+	my ( $self,$list ) = @_;
+	return $list->[ int( rand( scalar( @{ $list } ) ) ) ];
 }
 
 =encoding utf8
@@ -296,7 +383,7 @@ JSON::Schema::ToJSON - Generate example JSON structures from JSON Schema definit
 
 =head1 VERSION
 
-0.05
+0.06
 
 =head1 SYNOPSIS
 
@@ -316,8 +403,6 @@ JSON::Schema::ToJSON - Generate example JSON structures from JSON Schema definit
 L<JSON::Schema::ToJSON> is a class for generating "fake" or "example" JSON data
 structures from JSON Schema structures.
 
-Note this distribution is currently B<EXPERIMENTAL> and subject to breaking changes.
-
 =head1 CONSTRUCTOR ARGUMENTS
 
 =head2 example_key
@@ -357,30 +442,48 @@ Returns a randomly generated representative data structure that corresponds to t
 passed JSON schema. Can take either an already parsed JSON Schema or the raw JSON
 Schema string.
 
-=head1 BUGS, CAVEATS, AND GOTCHAS
+=head1 CAVEATS
 
-Bugs? Almost certainly.
+Caveats? The implementation is incomplete as using some of the more edge case JSON
+schema validation options may not generate representative JSON so they will not
+validate against the schema on a round trip. These include:
 
-Caveats? The implementation is currently incomplete, this is a work in progress so
-using some of the more edge case JSON schema validation options will not generate
-representative JSON so they will not validate against the schema on a round trip.
-These include:
+=over 4
 
-    additionalItems
-    patternProperties
-    additionalProperties
-    dependencies
-    allOf
-    anyOf
-    oneOf
-    not
+=item * additionalItems
+
+This is ignored
+
+=item * additionalProperties and patternProperties
+
+These are also ignored
+
+=item * dependencies
+
+This is *also* ignored, possible result of invalid JSON if used
+
+=item * oneOf
+
+Only the *first* schema from the oneOf list will be used (which means
+that the data returned may be invalid against others in the list)
+
+=item * not
+
+Currently any not restrictions are ignored as these can be very hand wavy
+but we will try a "best guess" in the case of "not" : { "type" : ... }
+
+=back
+
+In the case of oneOf and not the module will raise a warning to let you know that
+potentially invalid JSON has been generated. If you're using this module then you
+probably want to avoid oneOf and not in your schemas.
 
 It is also entirely possible to pass a schema that could never be validated, but
 will result in a generated structure anyway, example: an integer that has a "minimum"
 value of 2, "maximum" value of 4, and must be a "multipleOf" 5 - a nonsensical
 combination.
 
-Gotchas? The data generated is completely random, don't expect it to be the same
+Note that the data generated is completely random, don't expect it to be the same
 across runs or calls. The data is also meaningless in terms of what it represents
 such that an object property of "name" that is a string will be generated as, for
 example, "kj02@#fjs01je#$42wfjs" - The JSON generated is so you have a representative
